@@ -1,10 +1,15 @@
+import simplejson as json
+
+from sqlalchemy.ext.orderinglist import ordering_list
+import vdm.sqlalchemy
+
 from meta import *
 from types import make_uuid
-import vdm.sqlalchemy
-from sqlalchemy.ext.orderinglist import ordering_list
-
 from core import DomainObject, Package, package_table, Revision, State
 
+__all__ = ['PackageResource', 'package_resource_table',
+           'PackageResourceRevision', 'resource_revision_table']
+
 package_resource_table = Table(
     'package_resource', metadata,
     Column('id', types.UnicodeText, primary_key=True, default=make_uuid),
@@ -29,20 +34,22 @@ class PackageResource(vdm.sqlalchemy.RevisionedObjectMixin,
         self.format = format
         self.description = description
         self.hash = hash
-        
-    def __repr__(self):
-        return '<PackageResource id=%s package_id=%s url=%s description=%s>' % (self.id, self.package_id, self.url, self.description)
 
+    @property
+    def as_dict(self):
+        return dict([(col, getattr(self, col)) for col in self.get_columns()])
+        
     @staticmethod
     def get_columns():
-        return ('url', 'format', 'description', 'hash')
+        return ['url', 'format', 'description', 'hash']
 
 mapper(PackageResource, package_resource_table, properties={
     'package':orm.relation(Package,
-        backref=orm.backref('resources',
+        # all resources including deleted
+        backref=orm.backref('package_resources_all',
                             collection_class=ordering_list('position'),
-                            order_by=[package_resource_table.c.position],
                             cascade='all, delete, delete-orphan',
+                            order_by=package_resource_table.c.position,
                             ),
                        )
     },
@@ -53,3 +60,30 @@ mapper(PackageResource, package_resource_table, properties={
 vdm.sqlalchemy.modify_base_object_mapper(PackageResource, Revision, State)
 PackageResourceRevision= vdm.sqlalchemy.create_object_version(
     mapper, PackageResource, resource_revision_table)
+
+import vdm.sqlalchemy.stateful
+# TODO: move this into vdm
+def add_stateful_m21(object_to_alter, m21_property_name,
+        underlying_m21_attrname, identifier, **kwargs):
+    from sqlalchemy.orm import object_session
+    def _f(obj_to_delete):
+        sess = object_session(obj_to_delete)
+        if sess: # for tests at least must support obj not being sqlalchemy
+            sess.expunge(obj_to_delete)
+
+    active_list = vdm.sqlalchemy.stateful.DeferredProperty(
+            underlying_m21_attrname,
+            vdm.sqlalchemy.stateful.StatefulList,
+            # these args are passed to StatefulList
+            # identifier if url (could use id but have issue with None)
+            identifier=identifier,
+            unneeded_deleter=_f,
+            base_modifier=lambda x: x.get_as_of()
+            )
+    setattr(object_to_alter, m21_property_name, active_list)
+
+def package_resource_identifier(obj):
+    return json.dumps(obj.as_dict)
+add_stateful_m21(Package, 'resources', 'package_resources_all',
+                 package_resource_identifier)
+
