@@ -1,5 +1,8 @@
 from sqlalchemy import types
 
+def make_uuid():
+    return unicode(uuid.uuid4())
+
 import uuid
 class UuidType(types.TypeDecorator):
     impl = types.Unicode
@@ -27,8 +30,7 @@ class JsonType(types.TypeDecorator):
     impl = types.UnicodeText
 
     def process_bind_param(self, value, engine):
-        # None or {}
-        if not value: # ensure we stores nulls in db not json "null"
+        if value is None or value == {}: # ensure we stores nulls in db not json "null"
             return None
         else:
             # ensure_ascii=False => allow unicode but still need to convert
