@@ -5,12 +5,25 @@ from paste.fixture import AppError
 from ckan.tests import *
 import ckan.model as model
 from ckan.lib.create_test_data import CreateTestData
+import ckan.lib.helpers as h
+from genshi.core import escape as genshi_escape
 
 existing_extra_html = ('<label class="field_opt" for="Package-%(package_id)s-extras-%(key)s">%(capitalized_key)s</label>', '<input id="Package-%(package_id)s-extras-%(key)s" name="Package-%(package_id)s-extras-%(key)s" size="20" type="text" value="%(value)s">')
 
 package_form=''
 
-class TestPackageForm(TestController):
+class TestPackageBase(TestController):
+    key1 = u'key1 Less-than: < Umlaut: \xfc'
+    value1 = u'value1 Less-than: < Umlaut: \xfc'
+    # Note: Can't put a quotation mark in key1 or value1 because
+    # paste.fixture doesn't unescape the value in an input field
+    # on form submission. (But it works in real life.)
+    
+    def _assert_form_errors(self, res):
+        self.check_tag(res, '<form', 'class="has-errors"')
+        assert 'class="field_error"' in res, res
+
+class TestPackageForm(TestPackageBase):
     '''Inherit this in tests for these form testing methods'''
     def _check_package_read(self, res, **params):
         assert not 'Error' in res, res
@@ -26,9 +39,10 @@ class TestPackageForm(TestController):
         for res_index, values in self._get_resource_values(params['resources'], by_resource=True):
             self.check_named_element(main_div, 'tr', *values)
         assert params['notes'] in main_div, main_div_str
-        assert params['license'] in main_div, main_div_str
-        for tag in params['tags']:
-            assert tag.lower() in main_div, '%r %s' % (main_div_str, tag)
+        license = model.Package.get_license_register()[params['license_id']]
+        assert license.title in main_div, (license.title, main_div_str)
+        tag_names = [tag.lower() for tag in params['tags']]
+        self.check_named_element(main_div, 'ul', *tag_names)
         if params.has_key('state'):
             assert 'State: %s' % params['state'] in main_div.replace('</strong>', ''), main_div_str
         if isinstance(params['extras'], dict):
@@ -38,9 +52,20 @@ class TestPackageForm(TestController):
         else:
             raise NotImplementedError
         for key, value in extras:
-            self.check_named_element(main_div, 'tr', key, value)
+            key_in_html_body = self.escape_for_html_body(key)
+            value_in_html_body = self.escape_for_html_body(value)
+            self.check_named_element(main_div, 'tr', key_in_html_body, value_in_html_body)
+        if params.has_key('deleted_extras'):
+            if isinstance(params['deleted_extras'], dict):
+                deleted_extras = params['deleted_extras'].items()
+            elif isinstance(params['deleted_extras'], (list, tuple)):
+                deleted_extras = params['deleted_extras']
+            else:
+                raise NotImplementedError
+            for key, value in params['deleted_extras']:
+                self.check_named_element(main_div, 'tr', '!' + key)
+                self.check_named_element(main_div, 'tr', '!' + value)
 
-        
     def _check_preview(self, res, **params):
         preview =  str(res)[str(res).find('<div id="preview"'):str(res).find('<div id="footer">')]
         assert 'Preview' in preview, preview
@@ -52,21 +77,26 @@ class TestPackageForm(TestController):
             if isinstance(resource, (str, unicode)):
                 resource = [resource]
             self.check_named_element(preview, 'tr', resource[0], resource[1], resource[2], resource[3])
-        assert str(params['notes']) in preview, preview
-        assert str(params['license']) in preview, preview
-        tags_html_list = ['<a href="/tag/read/%s">%s</a>' % (str(tag), str(tag)) for tag in params['tags']]
-        tags_html_preview = ' '.join(tags_html_list)
-        assert tags_html_preview in preview, preview + tags_html_preview
+        preview_ascii = repr(preview)
+        assert str(params['notes']) in preview_ascii, preview_ascii
+        license = model.Package.get_license_register()[params['license_id']]
+        assert license.title in preview_ascii, (license.title, preview_ascii)
+        tag_names = [str(tag.lower()) for tag in params['tags']]
+        self.check_named_element(preview, 'ul', *tag_names)
         if params.has_key('state'):
             assert str(params['state']) in preview, preview
         else:
             assert 'state' not in preview
         for key, value in params['extras']:
-            self.check_named_element('td', key, value)
+            key_html = self.escape_for_html_body(key)
+            value_html = self.escape_for_html_body(value)
+            self.check_named_element(preview, 'tr', key_html, value_html)
         if params.has_key('deleted_extras'):
             for key, value in params['deleted_extras']:
-                self.check_named_element('td', '!' + key)
-                self.check_named_element('td', '!' + value)
+                key_html = self.escape_for_html_body(key)
+                value_html = self.escape_for_html_body(value)
+                self.check_named_element(preview, 'tr', '!' + key_html)
+                self.check_named_element(preview, 'tr', '!' + value_html)
 
     def _get_resource_values(self, resources, by_resource=False):
         assert isinstance(resources, (list, tuple))
@@ -91,10 +121,15 @@ class TestPackageForm(TestController):
             if by_resource:
                 yield(res_index, values)
 
+    def escape_for_html_body(self, unescaped_str):
+        # just deal with chars in tests
+        return unescaped_str.replace('<', '&lt;')
 
     def check_form_filled_correctly(self, res, **params):
         if params.has_key('pkg'):
             for key, value in params['pkg'].as_dict().items():
+                if key == 'license':
+                    key = 'license_id'
                 params[key] = value
         prefix = 'Package-%s-' % params['id']
         main_res = self.main_div(res)
@@ -105,11 +140,7 @@ class TestPackageForm(TestController):
         for res_index, res_field, expected_value in self._get_resource_values(params['resources']):
             self.check_tag(main_res, '%sresources-%i-%s' % (prefix, res_index, res_field), expected_value)
         self.check_tag_and_data(main_res, prefix+'notes', params['notes'])
-        if isinstance(params['license'], model.License):
-            license_ = params['license'].name
-        else:
-            license_ = params['license']
-        self.check_tag_and_data(main_res, 'selected', license_)
+        self.check_tag_and_data(main_res, 'selected', params['license_id'])
         if isinstance(params['tags'], (str, unicode)):
             tags = params['tags'].split()
         else:
@@ -123,8 +154,12 @@ class TestPackageForm(TestController):
         else:
             extras = params['extras']
         for key, value in extras:
-            self.check_tag_and_data(main_res, 'Package-%s-extras-%s' % (params['id'], key), key.capitalize())
-            self.check_tag(main_res, 'Package-%s-extras-%s' % (params['id'], key), value)
+            key_in_html_body = self.escape_for_html_body(key)
+            value_in_html_body = self.escape_for_html_body(value)
+            key_escaped = genshi_escape(key)
+            value_escaped = genshi_escape(value)
+            self.check_tag_and_data(main_res, 'Package-%s-extras-%s' % (params['id'], key_escaped), key_in_html_body.capitalize())
+            self.check_tag(main_res, 'Package-%s-extras-%s' % (params['id'], key_escaped), value_escaped)
         assert params['log_message'] in main_res, main_res
     
 
@@ -175,7 +210,7 @@ class TestReadOnly(TestPackageForm):
         self.check_tag_and_data(res, 'left arrow', '&lt;')
         self.check_tag_and_data(res, 'umlaut', u'\xfc')
         assert 'License:' in res
-        assert 'OKD Compliant::' in res
+        #assert 'OKD Compliant::' in res
         assert 'russian' in res
         assert 'david' in res
         assert 'roger' in res
@@ -271,14 +306,16 @@ class TestEdit(TestPackageForm):
         editpkg.add_tag_by_name(u'mytesttag')
         editpkg.add_resource(u'url escape: & umlaut: \xfc quote: "',
                              description=u'description escape: & umlaut: \xfc quote "')
-        model.Session.save(editpkg)
+        model.Session.add(editpkg)
         u = model.User(name=u'testadmin')
-        model.Session.save(u)
+        model.Session.add(u)
         model.repo.commit_and_remove()
 
         editpkg = model.Package.by_name(self.editpkg_name)
         admin = model.User.by_name(u'testadmin')
         model.setup_default_user_roles(editpkg, [admin])
+        model.repo.commit_and_remove()
+
         self.pkgid = editpkg.id
         offset = url_for(controller='package', action='edit', id=self.editpkg_name, package_form=package_form)
         self.res = self.app.get(offset)
@@ -297,15 +334,14 @@ class TestEdit(TestPackageForm):
 
     def test_edit(self):
         # the absolute basics
-        assert 'Packages - Edit' in self.res
+        assert 'Packages - Edit' in self.res, self.res
         assert self.editpkg.notes in self.res
 
         new_name = u'new-name'
         new_title = u'A Short Description of this Package'
         newurl = u'http://www.editpkgnewurl.com'
         new_download_url = newurl + u'/download/'
-        newlicense = u'Non-OKD Compliant::Other'
-        newlicenseid = model.License.by_name(newlicense).id
+        newlicense_id = u'cc-by'
         newversion = u'0.9b'
         fv = self.res.forms[0]
         prefix = 'Package-%s-' % self.pkgid
@@ -313,7 +349,7 @@ class TestEdit(TestPackageForm):
         fv[prefix + 'title'] =  new_title
         fv[prefix + 'url'] =  newurl
         fv[prefix + 'resources-0-url'] =  new_download_url
-        fv[prefix + 'license_id'] =  newlicenseid
+        fv[prefix + 'license_id'] =  newlicense_id
         fv[prefix + 'version'] = newversion
         res = fv.submit('commit')
         # get redirected ...
@@ -327,7 +363,7 @@ class TestEdit(TestPackageForm):
         assert pkg.url == newurl
         assert pkg.resources[0].url == new_download_url
         assert pkg.version == newversion
-        assert newlicense == pkg.license.name
+        assert newlicense_id == pkg.license.id
 
     def test_edit_2_not_groups(self):
         # not allowed to edit groups for now
@@ -390,15 +426,13 @@ u with umlaut \xc3\xbc
         assert 'Error' in res, res
         assert 'Name must be at least 2 characters long' in res, res
         # Ensure there is an error at the top of the form and by the field
-        assert 'class="form-errors"' in res, res
-        assert 'class="field_error"' in res, res
+        self._assert_form_errors(res)
 
         res = fv.submit('commit')
         assert 'Error' in res, res
         assert 'Name must be at least 2 characters long' in res, res
         # Ensure there is an error at the top of the form and by the field
-        assert 'class="form-errors"' in res, res
-        assert 'class="field_error"' in res, res
+        self._assert_form_errors(res)
 
     def test_missing_fields(self):
         # User edits and a field is left out in the commit parameters.
@@ -433,15 +467,16 @@ u with umlaut \xc3\xbc
         t2 = model.Tag(name=u'two')
         pkg.tags = [t1, t2]
         pkg.state = model.State.DELETED
-        pkg.license = model.License.by_name(u'OKD Compliant::Other')
+        pkg.license_id = u'other-open'
         extras = {'key1':'value1', 'key2':'value2', 'key3':'value3'}
         for key, value in extras.items():
             pkg.extras[unicode(key)] = unicode(value)
-        for obj in [pkg, t1, t2, pr1, pr2]:            
-            model.Session.save(obj)
+        for obj in [pkg, t1, t2, pr1, pr2]:
+            model.Session.add(obj)
         model.repo.commit_and_remove()
         pkg = model.Package.by_name(pkg_name)
         model.setup_default_user_roles(pkg, [self.admin])
+        model.repo.commit_and_remove()
 
         # Edit it
         offset = url_for(controller='package', action='edit', id=pkg.name, package_form=package_form)
@@ -461,12 +496,11 @@ u with umlaut \xc3\xbc
                      )
         assert len(resources[0]) == len(model.PackageResource.get_columns())
         notes = u'Very important'
-        license_id = 4
-        license = model.Session.query(model.License).get(license_id)
+        license_id = u'gpl-3.0'
         state = model.State.ACTIVE
         tags = (u'tag1', u'tag2', u'tag3')
         tags_txt = u' '.join(tags)
-        extra_changed = 'key1', 'value1 CHANGED'
+        extra_changed = 'key1', self.value1 + ' CHANGED'
         extra_new = 'newkey', 'newvalue'
         log_message = 'This is a comment'
         assert not model.Package.by_name(name)
@@ -483,9 +517,9 @@ u with umlaut \xc3\xbc
         fv[prefix+'license_id'] = license_id
         fv[prefix+'tags'] = tags_txt
         fv[prefix+'state'] = state
-        fv[prefix+'extras-%s' % extra_changed[0]] = extra_changed[1]
-        fv[prefix+'extras-newfield0-key'] = extra_new[0]
-        fv[prefix+'extras-newfield0-value'] = extra_new[1]
+        fv[prefix+'extras-%s' % extra_changed[0]] = extra_changed[1].encode('utf8')
+        fv[prefix+'extras-newfield0-key'] = extra_new[0].encode('utf8')
+        fv[prefix+'extras-newfield0-value'] = extra_new[1].encode('utf8')
         fv[prefix+'extras-key3-checkbox'] = True
         fv['log_message'] = log_message
         res = fv.submit('preview', extra_environ={'REMOTE_USER':'testadmin'})
@@ -499,7 +533,7 @@ u with umlaut \xc3\xbc
         self._check_preview(res, name=name, title=title, version=version,
                             url=url,
                             download_url='',
-                            resources=resources, notes=notes, license=license,
+                            resources=resources, notes=notes, license_id=license_id,
                             tags=tags, extras=current_extras,
                             deleted_extras=deleted_extras,
                             state=state)
@@ -508,7 +542,7 @@ u with umlaut \xc3\xbc
         self.check_form_filled_correctly(res, id=pkg.id, name=name,
                                          title=title, version=version,
                                          url=url, resources=resources,
-                                         notes=notes, license=license,
+                                         notes=notes, license_id=license_id,
                                          tags=tags, extras=current_extras,
                                          deleted_extras=deleted_extras,
                                          log_message=log_message,
@@ -524,8 +558,10 @@ u with umlaut \xc3\xbc
         self._check_package_read(res, name=name, title=title,
                                  version=version, url=url,
                                  resources=resources, notes=notes,
-                                 license=license, tags=tags,
-                                 extras=extras,
+                                 license_id=license_id, 
+                                 tags=tags,
+                                 extras=current_extras,
+                                 deleted_extras=deleted_extras,
                                  state=state,
                                  )
 
@@ -539,9 +575,12 @@ u with umlaut \xc3\xbc
             for field_index, res_field in enumerate(model.PackageResource.get_columns()):
                 assert getattr(pkg.resources[res_index], res_field) == resource[field_index]
         assert pkg.notes == notes
-        assert pkg.license_id == license_id
+        assert pkg.license.id == license_id
         saved_tagnames = [str(tag.name) for tag in pkg.tags]
-        assert saved_tagnames == list(tags)
+        saved_tagnames.sort()
+        expected_tagnames = list(tags)
+        expected_tagnames.sort()
+        assert saved_tagnames == expected_tagnames
         assert pkg.state == state
         assert len(pkg.extras) == len(current_extras)
         for key, value in current_extras:
@@ -555,6 +594,94 @@ u with umlaut \xc3\xbc
         exp_log_message = u'Creating package %s' % name
         #assert rev.message == exp_log_message
 
+    def test_edit_bad_log_message(self):
+        fv = self.res.forms[0]
+        prefix = 'Package-%s-' % self.pkgid
+        fv['log_message'] = u'Free enlargements: http://drugs.com/' # spam
+        res = fv.submit('preview')
+        assert 'Error' in res, res
+        assert 'No links are allowed' in res, res
+        self.check_tag(res, '<form', 'class="has-errors"')
+        assert 'No links are allowed' in res, res
+
+        res = fv.submit('commit')
+        assert 'Error' in res, res
+        self.check_tag(res, '<form', 'class="has-errors"')
+        assert 'No links are allowed' in res, res
+
+
+class TestMarkdownHtmlWhitelist(TestPackageForm):
+
+    pkg_name = u'markdownhtmlwhitelisttest'
+    pkg_notes = u'''
+<table width="100%" border="1">
+<tr>
+<td rowspan="2"><b>Description</b></td>
+<td rowspan="2"><b>Documentation</b></td>
+
+<td colspan="2"><b><center>Data -- Pkzipped</center></b> </td>
+</tr>
+<tr>
+<td><b>SAS .tpt</b></td>
+<td><b>ASCII CSV</b> </td>
+</tr>
+<tr>
+<td><b>Overview</b></td>
+<td><A HREF="http://www.nber.org/patents/subcategories.txt">subcategory.txt</A></td>
+<td colspan="2"><center>--</center></td>
+</tr>
+<script><!--
+alert('Hello world!');
+//-->
+</script>
+
+'''
+
+    def setup_method(self, method):
+        self.setUp()
+
+    def setUp(self):
+        model.Session.remove()
+        rev = model.repo.new_revision()
+        self.pkg = model.Package(name=self.pkg_name, notes=self.pkg_notes)
+        model.Session.add(self.pkg)
+        u = model.User(name=u'testadmin')
+        model.Session.add(u)
+        model.repo.commit_and_remove()
+
+        self.pkg = model.Package.by_name(self.pkg_name)
+        admin = model.User.by_name(u'testadmin')
+        model.setup_default_user_roles(self.pkg, [admin])
+        model.repo.commit_and_remove()
+        self.pkg_id = self.pkg.id
+        offset = url_for(controller='package', action='read', id=self.pkg_name)
+        self.res = self.app.get(offset)
+        model.repo.commit_and_remove()
+
+        self.pkg = model.Package.by_name(self.pkg_name)
+        self.admin = model.User.by_name(u'testadmin')
+
+    def teardown_method(self, method):
+        self.tearDown()
+
+    def tearDown(self):
+        model.repo.rebuild_db()
+        model.Session.remove()
+
+    def test_markdown_html_whitelist(self):
+        self.body = str(self.res)
+        self.assert_fragment('<table width="100%" border="1">')
+        self.assert_fragment('<td rowspan="2"><b>Description</b></td>')
+        self.assert_fragment('<a href="http://www.nber.org/patents/subcategories.txt">subcategory.txt</a>')
+        self.assert_fragment('<td colspan="2"><center>--</center></td>')
+        self.fail_if_fragment('<script>')
+
+    def assert_fragment(self, fragment):
+        assert fragment in self.body, (fragment, self.body)
+
+    def fail_if_fragment(self, fragment):
+        assert fragment not in self.body, (fragment, self.body)
+
 
 class TestNew(TestPackageForm):
     pkgname = u'testpkg'
@@ -609,7 +736,7 @@ class TestNew(TestPackageForm):
         pkg = model.Package.by_name(name)
         assert pkg
         assert pkg.name == name
-        assert pkg.resources == [], pkg.resources
+        assert not pkg.resources, pkg.resources
 
 
     def test_new(self):
@@ -633,16 +760,12 @@ class TestNew(TestPackageForm):
         res = fv.submit('preview')
         assert 'Error' in res, res
         assert 'Name must be at least 2 characters long' in res, res
-        # Ensure there is an error at the top of the form and by the field
-        assert 'class="form-errors"' in res, res
-        assert 'class="field_error"' in res, res
+        self._assert_form_errors(res)
 
         res = fv.submit('commit')
         assert 'Error' in res, res
         assert 'Name must be at least 2 characters long' in res, res
-        # Ensure there is an error at the top of the form and by the field
-        assert 'class="form-errors"' in res, res
-        assert 'class="field_error"' in res, res
+        self._assert_form_errors(res)
 
     def test_new_all_fields(self):
         name = u'test_name2'
@@ -651,11 +774,10 @@ class TestNew(TestPackageForm):
         url = u'http://something.com/somewhere.zip'
         download_url = u'http://something.com/somewhere-else.zip'
         notes = u'Very important'
-        license_id = 4
-        license = u'OKD Compliant::Creative Commons CCZero'
+        license_id = u'gpl-3.0'
         tags = (u'tag1', u'tag2', u'tag3', u'SomeCaps')
         tags_txt = u' '.join(tags)
-        extras = {'key1':'value1', 'key2':'value2', 'key3':'value3'}
+        extras = {self.key1:self.value1, 'key2':'value2', 'key3':'value3'}
         log_message = 'This is a comment'
         assert not model.Package.by_name(name)
         offset = url_for(controller='package', action='new', package_form=package_form)
@@ -673,8 +795,8 @@ class TestNew(TestPackageForm):
         fv[prefix+'license_id'] = license_id
         fv[prefix+'tags'] = tags_txt
         for i, extra in enumerate(extras.items()):
-            fv[prefix+'extras-newfield%s-key' % i] = extra[0]
-            fv[prefix+'extras-newfield%s-value' % i] = extra[1]
+            fv[prefix+'extras-newfield%s-key' % i] = extra[0].encode('utf8')
+            fv[prefix+'extras-newfield%s-value' % i] = extra[1].encode('utf8')
         fv['log_message'] = log_message
         res = fv.submit('preview')
         assert not 'Error' in res, res
@@ -685,7 +807,7 @@ class TestNew(TestPackageForm):
         self._check_preview(res, name=name, title=title, version=version,
                             url=url,
                             resources=resources_escaped, notes=notes,
-                            license=license,
+                            license_id=license_id,
                             tags=tags, extras=extras.items(),
                             )
 
@@ -693,14 +815,13 @@ class TestNew(TestPackageForm):
         self.check_form_filled_correctly(res, id='', name=name,
                                          title=title, version=version,
                                          url=url, resources=[download_url],
-                                         notes=notes, license=license,
+                                         notes=notes, license_id=license_id,
                                          tags=[tag.lower() for tag in tags],
                                          extras=extras,
 #                                         deleted_extras=deleted_extras,
                                          log_message=log_message,
 #                                         state=state
                                          )
-
         # Submit
         fv = res.forms[0]
         res = fv.submit('commit')
@@ -711,7 +832,8 @@ class TestNew(TestPackageForm):
         self._check_package_read(res, name=name, title=title,
                                  version=version, url=url,
                                  resources=[download_url], notes=notes,
-                                 license=license, tags=tags,
+                                 license_id=license_id, 
+                                 tags=tags,
                                  extras=extras,
 #                                 state=state,
                                  )
@@ -724,9 +846,12 @@ class TestNew(TestPackageForm):
         assert pkg.url == url
         assert pkg.resources[0].url == download_url
         assert pkg.notes == notes
-        assert pkg.license_id == license_id
+        assert pkg.license.id == license_id
         saved_tagnames = [str(tag.name) for tag in pkg.tags]
-        assert saved_tagnames == [tag.lower() for tag in list(tags)]
+        saved_tagnames.sort()
+        expected_tagnames = [tag.lower() for tag in tags]
+        expected_tagnames.sort()
+        assert saved_tagnames == expected_tagnames, '%r != %r' % (saved_tagnames, expected_tagnames)
         saved_groupnames = [str(group.name) for group in pkg.groups]
         assert len(pkg.extras) == len(extras)
         for key, value in extras.items():
@@ -765,9 +890,7 @@ class TestNew(TestPackageForm):
         res = fv.submit('commit')
         assert 'Error' in res, res
         assert 'Package name already exists in database' in res, res
-        # Ensure there is an error at the top of the form and by the field
-        assert 'class="form-errors"' in res, res
-        assert 'class="field_error"' in res, res
+        self._assert_form_errors(res)
         
     def test_missing_fields(self):
         # A field is left out in the commit parameters.
@@ -775,7 +898,9 @@ class TestNew(TestPackageForm):
         offset = url_for(controller='package', action='new', package_form=package_form)
         res = self.app.get(offset)
         assert 'Packages - New' in res
+        prefix = 'Package--'
         fv = res.forms[0]
+        fv[prefix + 'name'] = 'anything'
         del fv.fields['log_message']
         res = fv.submit('commit', status=400)
 
@@ -783,11 +908,31 @@ class TestNew(TestPackageForm):
         res = self.app.get(offset)
         assert 'Packages - New' in res
         fv = res.forms[0]
+        fv[prefix + 'name'] = 'anything'
         prefix = 'Package--'
-        del fv.fields[prefix + 'license_id']
+        del fv.fields[prefix + 'notes']
+        # NOTE Missing dropdowns fields don't cause KeyError in
+        # _serialized_value so don't register as an error here like
+        # text field tested here.
         res = fv.submit('commit', status=400)     
 
-class TestNewPreview(TestController):
+    def test_multi_resource_bug(self):
+        # ticket:276
+        offset = url_for(controller='package', action='new', package_form=package_form)
+        res = self.app.get(offset)
+        assert 'Packages - New' in res
+        fv = res.forms[0]
+        prefix = 'Package--'
+        fv[prefix + 'name'] = 'name276'
+        resformat = u'xls'    
+        fv[prefix + 'resources-0-format'] = resformat
+        res = fv.submit('preview')
+
+        res = self.main_div(res)
+        assert resformat in res, res
+        assert res.count(str(resformat)) == 1, res.count(str(resformat))
+
+class TestNewPreview(TestPackageBase):
     pkgname = u'testpkg'
     pkgtitle = u'mytesttitle'
 
@@ -821,7 +966,7 @@ class TestNewPreview(TestController):
         assert model.Session.query(model.Package).count() == 0, model.Session.query(model.Package).all()
         
 
-class TestNonActivePackages(TestController):
+class TestNonActivePackages(TestPackageBase):
 
     @classmethod
     def setup_class(self):
@@ -829,12 +974,13 @@ class TestNonActivePackages(TestController):
         self.non_active_name = u'test_nonactive'
         pkg = model.Package(name=self.non_active_name)
         model.repo.new_revision()
-        model.Session.save(pkg)
+        model.Session.add(pkg)
         model.repo.commit_and_remove()
 
         pkg = model.Session.query(model.Package).filter_by(name=self.non_active_name).one()
         admin = model.User.by_name(u'joeadmin')
         model.setup_default_user_roles(pkg, [admin])
+        model.repo.commit_and_remove()
         
         pkg = model.Session.query(model.Package).filter_by(name=self.non_active_name).one()
         pkg.delete() # becomes non active
@@ -874,7 +1020,7 @@ class TestNonActivePackages(TestController):
         assert '<strong>0</strong> packages found' in results_page, (self.non_active_name, results_page)
 
 
-class TestRevisions(TestController):
+class TestRevisions(TestPackageBase):
     @classmethod
     def setup_class(self):
         model.Session.remove()
@@ -885,7 +1031,7 @@ class TestRevisions(TestController):
         rev = model.repo.new_revision()
         self.pkg1 = model.Package(name=self.name)
         self.pkg1.notes = self.notes[0]
-        model.Session.save(self.pkg1)
+        model.Session.add(self.pkg1)
         model.setup_default_user_roles(self.pkg1)
         model.repo.commit_and_remove()
 
@@ -899,7 +1045,7 @@ class TestRevisions(TestController):
         self.pkg1 = model.Package.by_name(self.name)        
 
     @classmethod
-    def _teardown_class(self):
+    def teardown_class(self):
         rev = model.repo.new_revision()
         pkg1 = model.Package.by_name(self.name)
         pkg1.purge()
@@ -929,3 +1075,12 @@ class TestRevisions(TestController):
         assert 'Revision Differences' in main_res, main_res
         assert self.pkg1.name in main_res, main_res
         assert '<tr><td>notes</td><td><pre>- Written by Puccini\n+ Written off</pre></td></tr>' in main_res, main_res
+
+    def test_2_atom_feed(self):
+        offset = url_for(controller='package', action='history', id=self.pkg1.name)
+        offset = "%s?format=atom" % offset
+        res = self.app.get(offset)
+        assert '<feed' in res, res
+        assert 'xmlns="http://www.w3.org/2005/Atom"' in res, res
+        assert '</feed>' in res, res
+   
