@@ -15,7 +15,7 @@ def setup_config(command, filename, section, vars):
     
     from ckan import model
     log.info('Creating tables')
-    model.create_db()
-    model.init_db()
+    model.repo.create_db()
+    model.repo.init_db()
     log.info('Creating tables: SUCCESS')
 
